from django.conf import settings
from django.core.mail import EmailMultiAlternatives


def send(
    subject, md_full, html_full, sender, recipient, attachments={}
):
    msg = EmailMultiAlternatives(
        subject,
        md_full,
        sender or settings.DEFAULT_FROM_EMAIL,
        [recipient]
    )

    msg.attach_alternative(html_full, 'text/html')

    for filename, (data, mimetype) in attachments.items():
        if callable(data):
            msg.attach(filename, data(), mimetype)
        else:
            msg.attach(filename, data, mimetype)

    msg.send()



from django.conf import settings as site_settings


__all__ = [
    'CONTEXT_PROCESSORS',
    'MANAGEMENT_DOMAIN',
    'QUEUE'
]


QUEUE = getattr(site_settings, 'MAIL_RQ_QUEUE', 'default')
CONTEXT_PROCESSORS = getattr(site_settings, 'MAIL_CONTEXT_PROCESSORS', [])
MANAGEMENT_DOMAIN = getattr(
    site_settings,
    'MAIL_MANAGEMENT_DOMAIN',
    'example.com'
)



from . import settings
from importlib import import_module


class Message(object):
    recipient = ''
    subject = ''
    sender = ''
    tag = ''

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if not hasattr(self, key):  # pragma: no cover
                raise TypeError(
                    '__init__() got an unexpected keyword argument \'%s\'' % (
                        key
                    )
                )

            setattr(self, key, value)


def get_context(**kwargs):
    context = {}
    message = Message(**kwargs)

    for processor in settings.CONTEXT_PROCESSORS:
        module, func = processor.rsplit('.', 1)
        module = import_module(module)
        func = getattr(module, func)
        ctx = func(message)
        context.update(ctx)

    return context



from django.db import models
from hashlib import md5
from . import settings


class Tag(models.Model):
    recipient = models.EmailField(max_length=255)
    tag = models.CharField(max_length=36)
    hash = models.CharField(max_length=36)
    description = models.CharField(max_length=100, null=True, blank=True)
    unsubscribed = models.BooleanField(default=False)

    def __str__(self):
        return self.description or self.tag

    def save(self, *args, **kwargs):
        if not self.hash:
            self.hash = md5(
                (
                    '%s:%s' % (
                        self.tag,
                        self.recipient
                    )
                ).encode('utf-8')
            ).hexdigest()

        super().save(*args, **kwargs)

    def get_unsubscribe_url(self):
        return 'https://%s/email-tag/%s/unsubscribe/' % (
            settings.MANAGEMENT_DOMAIN,
            self.hash
        )

    class Meta:
        unique_together = ('tag', 'recipient')



from django.db import transaction
from django.template.loader import render_to_string, get_template
from django.utils.safestring import mark_safe
from markdown_deux import markdown
from mimetypes import guess_type
from .context import get_context
from .exceptions import InvalidTagError
from . import settings, tasks
import django_rq


__version__ = '1.2.0'


def send(
    recipient, subject, body_template, context,
    sender=None, tag=None, attachments={}
):
    from .models import Tag

    unsubscribe_url = None
    if tag:
        with transaction.atomic():
            if isinstance(tag, str):
                tag = {
                    'name': tag
                }

            if not tag.get('name'):
                raise InvalidTagError('Invalid email tag specified', tag)

            if Tag.objects.filter(
                recipient__iexact=recipient,
                tag=tag['name'],
                unsubscribed=True
            ).exists():
                return

            if not Tag.objects.filter(
                tag=tag['name'],
                recipient__iexact=recipient
            ).exists():
                tag = Tag.objects.create(
                    tag=tag['name'],
                    description=tag.get('description'),
                    recipient=recipient
                )
            else:
                tag = Tag.objects.get(
                    tag=tag['name'],
                    recipient=recipient
                )

            unsubscribe_url = tag.get_unsubscribe_url()

    base_context = get_context(
        recipient=recipient,
        subject=subject,
        sender=sender,
        tag=tag
    )

    md_body = render_to_string(
        body_template,
        dict(
            **base_context,
            **context
        )
    )

    mimetype, encoding = guess_type(body_template)
    if mimetype == 'text/html':
        html_body = md_body
    else:
        html_body = markdown(md_body)

    md_base = get_template('email/base.txt')
    html_base = get_template('email/base.html')

    md_full = md_base.render(
        dict(
            body=md_body,
            unsubscribe_url=unsubscribe_url,
            **base_context
        )
    )

    html_full = html_base.render(
        dict(
            body=mark_safe(html_body),
            unsubscribe_url=unsubscribe_url,
            **base_context
        )
    )

    queue = django_rq.get_queue(settings.QUEUE)
    queue.enqueue(
        tasks.send,
        args=[
            subject,
            md_full,
            html_full,
            sender,
            recipient,
            attachments
        ]
    )



# Generated by Django 2.1.2 on 2018-10-15 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('recipient', models.EmailField(max_length=255)),
                ('tag', models.CharField(max_length=36)),
                ('hash', models.CharField(max_length=36)),
                ('description', models.CharField(blank=True, max_length=100, null=True)),
                ('unsubscribed', models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together={('tag', 'recipient')},
        ),
    ]



from django.db import transaction
from django.http.response import Http404
from django.views.generic.base import TemplateView
from .models import Tag


class TagUnsubscribeView(TemplateView):
    template_name = 'email/tag.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        if self.request.method == 'GET':
            with transaction.atomic():
                try:
                    tag = Tag.objects.select_for_update().get(
                        hash=self.kwargs['hash']
                    )
                except Tag.DoesNotExist:
                    raise Http404('Tag matching query not found.')

                tag.unsubscribed = True
                tag.save()

            context['tag'] = tag

        return context



from django.conf.urls import url
from .views import TagUnsubscribeView


urlpatterns = [
    url(
        r'^email-tag/(?P<hash>.+)/unsubscribe/$',
        TagUnsubscribeView.as_view(),
        name='email_tag_unsubscribe'
    )
]
